"use strict";

var _imgDiffJs = require("img-diff-js");
var _md5File = _interopRequireDefault(require("md5-file"));
var _path = _interopRequireDefault(require("path"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// $FlowIgnore
// $FlowIgnore
var getMD5 = file => new Promise((resolve, reject) => {
  (0, _md5File.default)(file, (err, hash) => {
    if (err) reject(err);
    resolve(hash);
  });
});
var isPassed = ({
  width,
  height,
  diffCount,
  thresholdPixel,
  thresholdRate
}) => {
  if (typeof thresholdPixel === "number") {
    return diffCount <= thresholdPixel;
  } else if (typeof thresholdRate === "number") {
    var totalPixel = width * height;
    var ratio = diffCount / totalPixel;
    return ratio <= thresholdRate;
  }
  return diffCount === 0;
};
var createDiff = ({
  actualDir,
  expectedDir,
  diffDir,
  image,
  matchingThreshold,
  thresholdRate,
  thresholdPixel,
  enableAntialias
}) => {
  return Promise.all([getMD5(_path.default.join(actualDir, image)), getMD5(_path.default.join(expectedDir, image))]).then(([actualHash, expectedHash]) => {
    if (actualHash === expectedHash) {
      if (!process || !process.send) return;
      return process.send({
        passed: true,
        image
      });
    }
    var diffImage = image.replace(/\.[^\.]+$/, ".png");
    return (0, _imgDiffJs.imgDiff)({
      actualFilename: _path.default.join(actualDir, image),
      expectedFilename: _path.default.join(expectedDir, image),
      diffFilename: _path.default.join(diffDir, diffImage),
      options: {
        threshold: matchingThreshold,
        includeAA: !enableAntialias
      }
    }).then(({
      width,
      height,
      diffCount
    }) => {
      var passed = isPassed({
        width,
        height,
        diffCount,
        thresholdPixel,
        thresholdRate
      });
      if (!process || !process.send) return;
      process.send({
        passed,
        image
      });
    });
  });
};
process.on('message', data => {
  createDiff(data);
});